Package: simplifyEnrichment
Type: Package
Title: Simplify Functional Enrichment Results
Version: 2.6.0
Date: 2026-01-30
Authors@R: person("Zuguang", "Gu", email = "guzuguang@suat-sz.edu.cn", role = c("aut", "cre"),
                  comment = c('ORCID'="0000-0002-7395-8709"))
Depends: R (>= 4.1.0)
Imports: simona, ComplexHeatmap (>= 2.7.4), grid, circlize, GetoptLong,
        digest, tm, GO.db, AnnotationDbi, slam, methods, clue,
        grDevices, stats, utils, cluster (>= 1.14.2), colorspace,
        GlobalOptions (>= 0.1.0)
Suggests: knitr, ggplot2, cowplot, mclust, apcluster, MCL, dbscan,
        igraph, gridExtra, dynamicTreeCut, testthat, gridGraphics,
        flexclust, BiocManager, InteractiveComplexHeatmap (>= 0.99.11),
        shiny, shinydashboard, cola, hu6800.db, rmarkdown, genefilter,
        gridtext, fpc
Description: A new clustering algorithm, "binary cut", for clustering similarity matrices of functional terms
         is implemeted in this package. It also provides functions for visualizing, summarizing and comparing the clusterings.
biocViews: Software, Visualization, GO, Clustering, GeneSetEnrichment
URL: https://github.com/jokergoo/simplifyEnrichment,
        https://simplifyEnrichment.github.io
VignetteBuilder: knitr
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/simplifyEnrichment
git_branch: RELEASE_3_23
git_last_commit: 9a476a3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:43:54 UTC; biocbuild
Author: Zuguang Gu [aut, cre] (ORCID: <https://orcid.org/0000-0002-7395-8709>)
Maintainer: Zuguang Gu <guzuguang@suat-sz.edu.cn>
