

context("test GO_similarity")

# go_id = random_GO(100)
# m1 = GO_similarity(go_id)
# m2 = GO_similarity(go_id, mc.cores = 2)
# if(identical(topenv(), .GlobalEnv) {
# 	semData = env$semData	
# } else {
# 	semData = simplifyEnrichment:::env$semData
# }
# m3 = termSim(go_id, go_id, method = "Rel", semData = semData)

# test_that("test dend_noGO_similarityde_apply()", {
# 	expect_that(m1, identical(m3))
# 	expect_that(m2, identical(m3))
# })
